@@ -294,21 +294,24 @@ void container::append(node_ptr n)
 //n->validate();
 //#endif
 
-	if (n->m_parent != nullptr)
-		throw exception("attempt to append node that has already a parent");
-	if (n == nullptr)
-		throw exception("attempt to append nullptr node");
-	
-	if (m_child == nullptr)
+	if (n != nullptr)
 	{
-		m_last = m_child = n;
-		m_child->m_next = m_child->m_prev = nullptr;
-		n->parent(this);
-	}
-	else
-	{
-		m_last->insert_sibling(n, nullptr);
-		m_last = n;
+		if (n->m_parent != nullptr)
+			throw exception("attempt to append node that has already a parent");
+		if (n == nullptr)
+			throw exception("attempt to append nullptr node");
+		
+		if (m_child == nullptr)
+		{
+			m_last = m_child = n;
+			m_child->m_next = m_child->m_prev = nullptr;
+			n->parent(this);
+		}
+		else
+		{
+			m_last->insert_sibling(n, nullptr);
+			m_last = n;
+		}
 	}
 
 //#if DEBUG
@@ -324,26 +327,27 @@ void container::remove(node_ptr n)
 //n->validate();
 //#endif
 	
-	if (n->m_parent != this)
-		throw exception("attempt to remove node whose parent is invalid");
-	if (n == nullptr)
-		throw exception("attempt to remove nullptr node");
-	
-	if (m_child == n)
+	if (n != nullptr)
 	{
-		m_child = m_child->m_next;
-		if (m_child != nullptr)
-			m_child->m_prev = nullptr;
+		if (n->m_parent != this)
+			throw exception("attempt to remove node whose parent is invalid");
+		
+		if (m_child == n)
+		{
+			m_child = m_child->m_next;
+			if (m_child != nullptr)
+				m_child->m_prev = nullptr;
+			else
+				m_last = nullptr;
+			n->m_next = n->m_prev = n->m_parent = nullptr;
+		}
 		else
-			m_last = nullptr;
-		n->m_next = n->m_prev = n->m_parent = nullptr;
-	}
-	else
-	{
-		if (m_last == n)
-			m_last = n->m_prev;
-
-		m_child->remove_sibling(n);
+		{
+			if (m_last == n)
+				m_last = n->m_prev;
+	
+			m_child->remove_sibling(n);
+		}
 	}
 	
 //#if DEBUG
@@ -352,12 +356,27 @@ void container::remove(node_ptr n)
 //#endif
 }
 
-element_set container::find(const std::string& path) const
+//element_set container::find(const xpath& path) const
+//{
+//	return path.evaluate<element>(*this);
+//}
+//
+//element* container::find_first(const xpath& path) const
+//{
+//	element_set s = path.evaluate<element>(*this);
+//	
+//	element* result = nullptr;
+//	if (not s.empty())
+//		result = s.front();
+//	return result;
+//}
+
+element_set container::find(const char* path) const
 {
 	return xpath(path).evaluate<element>(*this);
 }
 
-element* container::find_first(const std::string& path) const
+element* container::find_first(const char* path) const
 {
 	element_set s = xpath(path).evaluate<element>(*this);
 	
@@ -367,6 +386,26 @@ element* container::find_first(const std::string& path) const
 	return result;
 }
 
+void container::find(const char* path, node_set& nodes) const
+{
+	nodes = xpath(path).evaluate<node>(*this);
+}
+
+void container::find(const char* path, element_set& elements) const
+{
+	elements = xpath(path).evaluate<element>(*this);
+}
+
+node* container::find_first_node(const char* path) const
+{
+	node_set s = xpath(path).evaluate<node>(*this);
+	
+	node* result = nullptr;
+	if (not s.empty())
+		result = s.front();
+	return result;
+}
+
 container::size_type container::size() const
 {
 	size_type result = 0;
@@ -540,7 +579,14 @@ container::node_iterator container::insert(node* position, node* n)
 //	position->validate();
 //#endif
 	
-	if (m_child == position)	// n becomes the first in the list
+	if (m_child == nullptr)
+	{
+		if (position != nullptr) throw exception("invalid position for empty container");
+		m_child = n;
+		m_child->m_next = m_child->m_prev = nullptr;
+		n->parent(this);
+	}
+	else if (m_child == position)	// n becomes the first in the list
 	{
 		n->parent(this);
 		n->m_next = m_child;
@@ -587,66 +633,6 @@ void container::validate()
 	}
 }
 
-//void container::private_insert(node* position, node* n)
-//{
-//	if (n->m_parent != nullptr or n->m_next != nullptr or n->m_prev != nullptr)
-//		throw exception("attempt to insert a node that has parent and/or next/prev");
-//	
-//	if (position == nullptr)
-//		push_back(n);
-//	else
-//	{
-//		if (position->m_parent != this)
-//			throw exception("position is not a child node of this container");
-//
-//		node* child = m_child;
-//		while (child != nullptr and child != position)
-//			child = child->m_next;
-//		if (child == nullptr)
-//			throw zeep::exception("position is not a valid child node");
-//		
-//		n->parent(this);
-//		n->m_next = position;
-//		position->m_prev = n;
-//
-//		n->m_prev = position->m_prev;
-//		if (n->m_prev != nullptr)
-//			n->m_prev->m_next = n;
-//		
-//		if (position == m_child)
-//			m_child = n;
-//	}
-//}
-//
-//void container::private_erase(node* n)
-//{
-////	if (n == nullptr)
-////		throw exception("attempt to erase nullptr node");
-////	
-////	if (n->m_parent != this)
-////		throw exception("cannot erase node, invalid parent");
-////	
-////	if (m_child == n)
-////	{
-////		m_child = m_child->m_next;
-////		if (m_child != nullptr)
-////			m_child->m_prev = nullptr;
-////	}
-////	else
-////		m_child->remove_sibling(n);
-////	
-////	if (n == m_last)
-////	{
-////		m_last = m_child;
-////		while (m_last->m_next != nullptr)
-////			m_last = m_last->m_next;
-////	}
-////	
-////	n->m_next = n->m_prev = n->m_parent = nullptr;
-//	remove(n);
-//	delete n;
-//}
-
 // --------------------------------------------------------------------
 // root_node
 
@@ -700,7 +686,8 @@ void root_node::child_element(element* child)
 		delete e;
 	}
 	
-	container::append(child);
+	if (child != nullptr)
+		container::append(child);
 }
 
 void root_node::write(writer& w) const
@@ -733,13 +720,15 @@ bool root_node::equals(const node* n) const
 
 void root_node::append(node* n)
 {
-	if (dynamic_cast<processing_instruction*>(n) == nullptr and
+	if (dynamic_cast<element*>(n) != nullptr and child_element() == nullptr)
+		child_element(static_cast<element*>(n));
+	else if (dynamic_cast<processing_instruction*>(n) == nullptr and
 		dynamic_cast<comment*>(n) == nullptr)
 	{
 		throw exception("can only append comment and processing instruction nodes to a root_node");
 	}
-	
-	container::append(n);
+	else
+		container::append(n);
 }
 
 // --------------------------------------------------------------------
