@@ -9,6 +9,8 @@
 
 #include <zeep/config.hpp>
 
+#include <iomanip>
+
 #include <boost/foreach.hpp>
 #define foreach BOOST_FOREACH
 #include <boost/algorithm/string.hpp>
@@ -24,6 +26,27 @@ namespace ba = boost::algorithm;
 namespace zeep { namespace http {
 namespace el
 {
+	
+#define ZEEP_DEFINE_AS_INT(T) \
+template<> T object::as<T>() const					\
+{													\
+	T result = 0;									\
+													\
+	if (m_impl)										\
+		result = static_cast<T>(m_impl->to_int());	\
+													\
+	return result;									\
+}
+
+ZEEP_DEFINE_AS_INT(int8)
+ZEEP_DEFINE_AS_INT(uint8)
+ZEEP_DEFINE_AS_INT(int16)
+ZEEP_DEFINE_AS_INT(uint16)
+ZEEP_DEFINE_AS_INT(int32)
+ZEEP_DEFINE_AS_INT(uint32)
+ZEEP_DEFINE_AS_INT(int64)
+ZEEP_DEFINE_AS_INT(uint64)	
+	
 namespace detail
 {
 
@@ -42,6 +65,11 @@ string object_impl::to_str() const
 	throw exception("cannot convert to requested type");
 }
 
+string object_impl::to_JSON() const
+{
+	throw exception("cannot convert to JSON string");
+}
+
 object& base_array_object_impl::at(uint32 ix)
 {
 	throw exception("method 'at' not implemented");
@@ -61,16 +89,28 @@ class int_object_impl : public detail::object_impl
 						: detail::object_impl(object::number_type)
 						, m_v(v)	{}
 
-	virtual void	print(ostream& os) const		{ os << m_v; }
+	virtual void	print(ostream& os) const			{ os << m_v; }
 	virtual int		compare(object_impl* rhs) const;
 
 	virtual int64	to_int() const						{ return m_v; }
 	virtual double	to_double() const					{ return static_cast<double>(m_v); }
 	virtual string	to_str() const						{ return boost::lexical_cast<string>(m_v); }
+	virtual string	to_JSON() const						{ return boost::lexical_cast<string>(m_v); }
 
 	int64			m_v;
 };
 
+class bool_object_impl : public int_object_impl
+{
+  public:
+					bool_object_impl(bool v)
+						: int_object_impl(object::number_type) {}
+
+	virtual void	print(ostream& os) const			{ os << (m_v ? "true" : "false"); }
+	virtual string	to_str() const						{ return m_v ? "true" : "false"; }
+	virtual string	to_JSON() const						{ return m_v ? "true" : "false"; }
+};
+
 class float_object_impl : public detail::object_impl
 {
   public:
@@ -84,6 +124,7 @@ class float_object_impl : public detail::object_impl
 	virtual int64	to_int() const						{ return static_cast<int64>(tr1::round(m_v)); }
 	virtual double	to_double() const					{ return m_v; }
 	virtual string	to_str() const						{ return boost::lexical_cast<string>(m_v); }
+	virtual string	to_JSON() const						{ return boost::lexical_cast<string>(m_v); }
 	
 	double			m_v;
 };
@@ -101,6 +142,34 @@ class string_object_impl : public detail::object_impl
 	virtual int64	to_int() const						{ return boost::lexical_cast<int64>(m_v); }
 	virtual double	to_double() const					{ return boost::lexical_cast<double>(m_v); }
 	virtual string	to_str() const						{ return m_v; }
+	virtual string	to_JSON() const
+					{
+						stringstream s;
+						s << '"';
+						foreach (char ch, m_v)
+						{
+							switch (ch)
+							{
+								case '"':	s << "\\\""; break;
+								case '\\':	s << "\\\\"; break;
+								case '/':	s << "\\/"; break;
+								case 010:	s << "\\b"; break;
+								case '\t':	s << "\\t"; break;
+								case '\n':	s << "\\n"; break;
+								case 014:	s << "\\f"; break;
+								case '\r':	s << "\\r"; break;
+								default:
+								{
+									if ((unsigned int)(ch) < 040)
+										s << "\\u" << hex << setfill('0') << setw(4) << int(ch);
+									else
+										s << ch;
+								}
+							}
+						}
+						s << '"';
+						return s.str();
+					}
 	
 	string			m_v;
 };
@@ -150,15 +219,15 @@ class vector_object_impl : public detail::base_array_object_impl
 						return m_v[ix];
 					}
 
-	virtual object&	operator[](uint32 ix)
+	virtual object&	operator[](const object& index)
 					{
-						return m_v[ix];
+						return m_v[index.as<uint32>()];
 					}
 
 	virtual const object
-					operator[](uint32 ix) const
+					operator[](const object& index) const
 					{
-						return m_v[ix];
+						return m_v[index.as<uint32>()];
 					}
 
 	virtual void	print(ostream& os) const
@@ -186,6 +255,20 @@ class vector_object_impl : public detail::base_array_object_impl
 							return new vector_object_iterator_impl(m_v.end());
 					}
 	
+	virtual string	to_JSON() const
+					{
+						stringstream s;
+						s << '[';
+						for (uint32 ix = 0; ix < m_v.size(); ++ix)
+						{
+							if (ix > 0)
+								s << ',';
+							s << m_v[ix].toJSON();
+						}
+						s << ']';
+						return s.str();
+					}
+
 	vector<object>	m_v;
 };
 
@@ -237,6 +320,21 @@ class struct_object_impl : public detail::base_struct_object_impl
 						return result;
 					}
 
+	
+	virtual string	to_JSON() const
+					{
+						stringstream s;
+						s << '{';
+						for (map<string,object>::const_iterator o = m_v.begin(); o != m_v.end(); ++o)
+						{
+							if (o != m_v.begin())
+								s << ',';
+							s << '"' << o->first << '"' << ':' << o->second.toJSON();
+						}
+						s << '}';
+						return s.str();
+					}
+
   private:
 	map<string,object>
 					m_v;
@@ -422,6 +520,7 @@ object::object(const vector<string>& v)
 }
 
 object::object(bool v)
+//	: m_impl(new bool_object_impl(v))
 	: m_impl(new int_object_impl(v))
 {
 }
@@ -508,6 +607,7 @@ object& object::operator=(bool v)
 {
 	if (m_impl != nullptr)
 		m_impl->release();
+//	m_impl = new bool_object_impl(v);
 	m_impl = new int_object_impl(v);
 	return *this;
 }
@@ -636,34 +736,13 @@ bool object::empty() const
 		case null_type:		result = true; break;
 		case string_type:	result = m_impl->to_str().empty(); break;
 		case array_type:	result = static_cast<detail::base_array_object_impl*>(m_impl)->count() == 0; break;
-		default:			result = true; break;
+		case number_type:	result = false; break;
+		case struct_type:	result = false; break;
 	}
 	
 	return result;
 }
 
-#define ZEEP_DEFINE_AS_INT(T) \
-template<> T object::as<T>() const					\
-{													\
-	T result = 0;									\
-													\
-	if (m_impl)										\
-		result = static_cast<T>(m_impl->to_int());	\
-													\
-	return result;									\
-}
-
-ZEEP_DEFINE_AS_INT(int8)
-ZEEP_DEFINE_AS_INT(uint8)
-ZEEP_DEFINE_AS_INT(int16)
-ZEEP_DEFINE_AS_INT(uint16)
-ZEEP_DEFINE_AS_INT(int32)
-ZEEP_DEFINE_AS_INT(uint32)
-ZEEP_DEFINE_AS_INT(int64)
-ZEEP_DEFINE_AS_INT(uint64)
-
-//ZEEP_DEFINE_AS_INT(uint64)
-
 template<> string object::as<string>() const
 {
 	string result;
@@ -722,46 +801,70 @@ const object object::operator[](const char* name) const
 	return object();
 }
 
-const object object::operator[](uint32 ix) const
+const object object::operator[](const object& index) const
 {
 	object result;
 
-	const detail::base_array_object_impl* impl = dynamic_cast<const detail::base_array_object_impl*>(m_impl);
-	if (impl != nullptr and ix < impl->count())
-		result = impl->at(ix);
+	switch (type())
+	{
+		case array_type:
+		{
+			const detail::base_array_object_impl* impl = static_cast<const detail::base_array_object_impl*>(m_impl);
+			if (impl != nullptr and index.as<uint32>() < impl->count())
+				result = impl->at(index.as<uint32>());
+			break;
+		}
+		
+		case struct_type:
+		{
+			const detail::base_struct_object_impl* impl = static_cast<const detail::base_struct_object_impl*>(m_impl);
+			result = impl->field(index.as<string>());
+			break;
+		}
+		
+		default:
+			; // return null
+	}
 	
 	return result;
 }
 
 object& object::operator[](const string& name)
 {
-	struct_object_impl* impl = dynamic_cast<struct_object_impl*>(m_impl);
-	
-	if (impl == nullptr)
-	{
-		if (m_impl != nullptr)
-			m_impl->release();
-		m_impl = impl = new struct_object_impl();
-	}
-
-	return impl->field(name);
+	if (name.empty())
+		throw exception("invalid empty name for structure object");
+	return operator[](object(name));
 }
 
 object& object::operator[](const char* name)
 {
 	if (name == nullptr)
 		throw exception("invalid empty name for structure object");
-	return operator[](string(name));
+	return operator[](object(name));
 }
 
-object& object::operator[](uint32 ix)
+object& object::operator[](const object& index)
 {
-	detail::base_array_object_impl* impl = dynamic_cast<detail::base_array_object_impl*>(m_impl);
-	if (impl != nullptr and ix < impl->count())
-		return impl->at(ix);
-	
-	throw exception("index out of bounds or object is not an array");
-	return *this;
+	if (type() == array_type)
+	{
+		vector_object_impl* impl = static_cast<vector_object_impl*>(m_impl);
+		return impl->at(index.as<uint32>());
+	}
+	else if (type() == struct_type)
+	{
+		struct_object_impl* impl = static_cast<struct_object_impl*>(m_impl);
+		return impl->field(index.as<string>());
+	}
+	else
+	{
+		if (m_impl != nullptr)
+			m_impl->release();
+
+		struct_object_impl* impl;
+		m_impl = impl = new struct_object_impl();
+
+		return impl->field(index.as<string>());	
+	}
 }
 
 bool object::operator<(const object& rhs) const
@@ -814,11 +917,19 @@ ostream& operator<<(ostream& os, const object& o)
 	return os;
 }
 
+string object::toJSON() const
+{
+	string result;
+	if (m_impl != nullptr)
+		result = m_impl->to_JSON();
+	return result;
+}
+
 object operator+(const object& a, const object& b)
 {
 	object result;
 	
-	if (typeid(*a.m_impl) == typeid(*b.m_impl))
+	if (a.m_impl != nullptr and b.m_impl != nullptr and typeid(*a.m_impl) == typeid(*b.m_impl))
 	{
 		if (dynamic_cast<int_object_impl*>(a.m_impl))
 			result = static_cast<int_object_impl*>(a.m_impl)->m_v + static_cast<int_object_impl*>(b.m_impl)->m_v;
@@ -865,7 +976,7 @@ object operator*(const object& a, const object& b)
 
 object operator/(const object& a, const object& b)
 {
-	return object(a.as<double>() * b.as<double>());
+	return object(a.as<double>() / b.as<double>());
 }
 
 object operator%(const object& a, const object& b)
@@ -1115,8 +1226,7 @@ void interpreter::get_next_token()
 		els_NumberFraction,
 		els_Name,
 		els_Literal
-	} start, state;
-	start = state = els_Start;
+	} state = els_Start;
 
 	token_type token = elt_undef;
 	double fraction = 1.0;
@@ -1437,7 +1547,7 @@ object interpreter::parse_primary_expr()
 				{
 					match(m_lookahead);
 					if (result.type() == object::array_type and (m_token_string == "count" or m_token_string == "length"))
-						result = object(result.count());
+						result = object((uint32)result.count());
 					else
 						result = const_cast<const object&>(result)[m_token_string];
 					match(elt_object);
@@ -1448,11 +1558,13 @@ object interpreter::parse_primary_expr()
 				{
 					match(m_lookahead);
 					
-					uint32 index = static_cast<uint32>(m_token_number);
-					match(elt_number);
+					object index = parse_expr();
 					match(elt_rbracket);
 					
-					result = const_cast<object&>(result)[index];
+					if (index.empty() or (result.type() != object::array_type and result.type() != object::struct_type))
+						result = object();
+					else
+						result = const_cast<object&>(result)[index];
 					continue;
 				}
 
@@ -1512,6 +1624,8 @@ void evaluate_el(
 		result = interpreter.evaluate(m[1]);
 //cerr << "evaluated \"" << text << "\" => \"" << result << '"' << endl;
 	}
+	else
+		result = text;
 }
 
 bool evaluate_el(
@@ -1605,8 +1719,6 @@ const request& scope::get_request() const
 	return *m_req;
 }
 
-
-
 }
 }
 }
