@@ -65,7 +65,8 @@ void webapp::handle_request(
 	string uri = req.uri;
 
 	// shortcut, only handle GET, POST and PUT
-	if (req.method != "GET" and req.method != "POST" and req.method != "PUT")
+	if (req.method != "GET" and req.method != "POST" and req.method != "PUT" and
+		req.method != "OPTIONS" and req.method != "HEAD")
 	{
 		rep = reply::stock_reply(bad_request);
 		return;
@@ -111,21 +112,36 @@ void webapp::handle_request(
 		if (s != string::npos)
 			uri.erase(s, string::npos);
 		scope.put("baseuri", uri);
-		
-		init_scope(scope);
-		
+		scope.put("mobile", req.is_mobile());
+
 		handler_map::iterator handler = m_dispatch_table.find(uri);
 		if (handler == m_dispatch_table.end())
 			handler = m_dispatch_table.find(action);
 
 		if (handler != m_dispatch_table.end())
-			handler->second(req, scope, rep);
+		{
+			if (req.method == "OPTIONS")
+			{
+				rep = reply::stock_reply(ok);
+				rep.set_header("Allow", "GET,HEAD,POST,OPTIONS");
+				rep.set_content("", "text/plain");
+			}
+			else
+			{
+				init_scope(scope);
+				
+				handler->second(req, scope, rep);
+				
+				if (req.method == "HEAD")
+					rep.set_content("", rep.get_content_type());
+			}
+		}
 		else
 			throw not_found;
 	}
 	catch (unauthorized_exception& e)
 	{
-		create_unauth_reply(e.m_stale, rep);
+		create_unauth_reply(e.m_stale, e.m_realm, rep);
 	}
 	catch (status_type& s)
 	{
@@ -140,7 +156,7 @@ void webapp::handle_request(
 	}
 }
 
-void webapp::create_unauth_reply(bool stale, reply& rep)
+void webapp::create_unauth_reply(bool stale, const string& realm, reply& rep)
 {
 	rep = reply::stock_reply(unauthorized);
 }
@@ -176,8 +192,6 @@ void webapp::handle_file(
 
 			local_date_time fileDate(from_time_t(fs::last_write_time(file)), time_zone_ptr());
 
-			cerr << file << endl << modifiedSince << endl << fileDate << endl << endl;
-
 			if (fileDate <= modifiedSince)
 			{
 				reply = zeep::http::reply::stock_reply(zeep::http::not_modified);
@@ -205,6 +219,8 @@ void webapp::handle_file(
 		mimetype = "image/svg+xml";
 	else if (file.extension() == ".html" or file.extension() == ".htm")
 		mimetype = "text/html";
+	else if (file.extension() == ".xml" or file.extension() == ".xsl" or file.extension() == ".xslt")
+		mimetype = "text/xml";
 	else if (file.extension() == ".xhtml")
 		mimetype = "application/xhtml+xml";
 
@@ -221,9 +237,6 @@ void webapp::handle_file(
 	s << t2;
 
 	reply.set_header("Last-Modified", s.str());
-
-
-cerr << s.str() << endl << endl;
 }
 
 void webapp::load_template(
@@ -232,7 +245,40 @@ void webapp::load_template(
 {
 	fs::ifstream data(m_docroot / file, ios::binary);
 	if (not data.is_open())
-		throw exception((boost::format("file not found: '%1%'") % (m_docroot / file)).str());
+	{
+		if (not fs::exists(m_docroot))
+			throw exception((boost::format("configuration error, docroot not found: '%1%'") % m_docroot).str());
+		else
+		{
+#if defined(_MSC_VER)
+			char msg[1024] = "";
+
+		    DWORD dw = ::GetLastError();
+			if (dw != NO_ERROR)
+			{
+			    char* lpMsgBuf;
+				int m = ::FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
+					NULL, dw, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPSTR)&lpMsgBuf, 0, NULL);
+			
+				if (lpMsgBuf != nullptr)
+				{
+					// strip off the trailing whitespace characters
+					while (m > 0 and isspace(lpMsgBuf[m - 1]))
+						--m;
+					lpMsgBuf[m] = 0;
+
+					strncpy(msg, lpMsgBuf, sizeof(msg));
+		
+					::LocalFree(lpMsgBuf);
+				}
+			}
+
+			throw exception((boost::format("error opening: %1% (%2%)") % (m_docroot / file) % msg).str());
+#else
+			throw exception((boost::format("error opening: %1% (%2%)") % (m_docroot / file) % strerror(errno)).str());
+#endif
+		}
+	}
 	doc.read(data);
 }
 
@@ -555,6 +601,13 @@ void webapp::process_option(
 	fs::path			dir)
 {
 	string value = node->get_attribute("value");
+	if (not value.empty())
+	{
+		el::object o;
+		evaluate_el(scope, value, o);
+		value = o.as<string>();
+	}
+
 	string selected = node->get_attribute("selected");
 	if (not selected.empty())
 	{
@@ -587,6 +640,12 @@ void webapp::process_checkbox(
 	fs::path			dir)
 {
 	string name = node->get_attribute("name");
+	if (not name.empty())
+	{
+		el::object o;
+		evaluate_el(scope, name, o);
+		name = o.as<string>();
+	}
 
 	bool checked = false;
 	if (not node->get_attribute("checked").empty())
@@ -732,7 +791,7 @@ void webapp::get_parameters(
 
 	while (not ps.empty())
 	{
-		string::size_type e = ps.find('&');
+		string::size_type e = ps.find_first_of("&;");
 		string param;
 		
 		if (e != string::npos)
