@@ -1593,7 +1593,9 @@ object union_expression::evaluate(expression_context& context)
 struct xpath_imp
 {
 						xpath_imp();
-						~xpath_imp();
+
+	void				reference();
+	void				release();
 	
 	node_set			evaluate(node& root, context_imp& context);
 
@@ -1644,17 +1646,34 @@ struct xpath_imp
 
 	// the generated expression
 	expression_ptr		m_expr;
+
+  private:
+						~xpath_imp();
+
+	long				m_refcount;
 };
 
 // --------------------------------------------------------------------
 
 xpath_imp::xpath_imp()
+	: m_refcount(1)
 {
 }
 
 xpath_imp::~xpath_imp()
 {
-}	
+}
+
+void xpath_imp::reference()
+{
+	++m_refcount;
+}
+
+void xpath_imp::release()
+{
+	if (--m_refcount <= 0)
+		delete this;
+}
 
 void xpath_imp::parse(const string& path)
 {
@@ -1897,13 +1916,12 @@ Token xpath_imp::get_next_token()
 		xps_QName,
 		xps_QName2,
 		xps_Literal
-	} start, state;
-	start = state = xps_Start;
+	} state = xps_Start;
 
 	Token token = xp_Undef;
 	bool variable = false;
 	double fraction = 1.0;
-	unicode quoteChar;
+	unicode quoteChar = 0;
 
 	m_token_string.clear();
 	
@@ -2616,9 +2634,37 @@ xpath::xpath(const string& path)
 	m_impl->parse(path);
 }
 
+xpath::xpath(const char* path)
+	: m_impl(new xpath_imp())
+{
+	string p;
+	if (path != nullptr)
+		p = path;
+	
+	m_impl->parse(p);
+}
+
+xpath::xpath(const xpath& rhs)
+	: m_impl(rhs.m_impl)
+{
+	m_impl->reference();
+}
+
+xpath& xpath::operator=(const xpath& rhs)
+{
+	if (this != &rhs)
+	{
+		m_impl->release();
+		m_impl = rhs.m_impl;
+		m_impl->reference();
+	}
+	
+	return *this;
+}
+
 xpath::~xpath()
 {
-	delete m_impl;
+	m_impl->release();
 }
 
 template<>
@@ -2657,5 +2703,27 @@ element_set xpath::evaluate<element>(const node& root, context& ctxt) const
 	return result;
 }
 
+bool xpath::matches(const node* n) const
+{
+	bool result = false;
+	if (n != nullptr)
+	{
+		const node* root = n->root();
+	
+		context ctxt;
+		object s(m_impl->evaluate(const_cast<node&>(*root), *ctxt.m_impl));
+		foreach (node* e, s.as<const node_set&>())
+		{
+			if (e == n)
+			{
+				result = true;
+				break;
+			}
+		}
+	}
+	
+	return result;
+}
+
 }
 }
